@@ -52,9 +52,14 @@ module Advent (
   , Part(..)
   , Day(..)
   , NextDayTime(..)
+  , DayStats(..)
+  , Stats
   , AoCOpts(..)
   , AoCUserAgent(..)
   , SubmitRes(..), showSubmitRes
+  , statsForDayPart
+  , inferSubmitRes
+  , inferSubmitRes_
   , runAoC
   , runAoC_
   , defaultAoCOpts
@@ -243,6 +248,15 @@ data AoC :: Type -> Type where
     AoCNextDayTime
         :: AoC NextDayTime
 
+    -- | Fetch the per-day completion stats for the current year.  Does not
+    -- require a session key.
+    --
+    -- /Cacheing rules/: Not cached.
+    --
+    -- @since 0.2.11.0
+    AoCStats
+        :: AoC Stats
+
 deriving instance Show (AoC a)
 deriving instance Typeable (AoC a)
 
@@ -255,6 +269,7 @@ aocDay (AoCLeaderboard _) = Nothing
 aocDay (AoCDailyLeaderboard d) = Just d
 aocDay AoCGlobalLeaderboard = Nothing
 aocDay AoCNextDayTime       = Nothing
+aocDay AoCStats             = Nothing
 
 -- | A possible (syncronous, logical, pure) error returnable from 'runAoC'.
 -- Does not cover any asynchronous or IO errors.
@@ -306,6 +321,10 @@ data AoCOpts = AoCOpts
       -- | Throttle delay, in milliseconds.  Minimum is 1000000.  Default
       -- is 3000000 (3 seconds).
     , _aThrottle   :: Int
+      -- | Attempt to infer submission rank by fetching stats after a
+      -- correct submission that doesn't include rank information.
+      -- Default is False.
+    , _aInferRankOnSubmission :: Bool
     }
   deriving (Show, Typeable, Generic)
 
@@ -326,6 +345,7 @@ defaultAoCOpts aua y s = AoCOpts
     , _aCache      = Nothing
     , _aForce      = False
     , _aThrottle   = 3000000
+    , _aInferRankOnSubmission = False
     }
 
 -- | HTTPS base of Advent of Code API.
@@ -339,13 +359,15 @@ aocReq aua yr = \case
     AoCInput  i       -> let _ :<|> r :<|> _ = adventAPIPuzzleClient aua yr i in r
     AoCSubmit i p ans -> let _ :<|> _ :<|> r = adventAPIPuzzleClient aua yr i
                          in  r (SubmitInfo p ans) <&> \(x :<|> y) -> (x, y)
-    AoCLeaderboard c  -> let _ :<|> _ :<|> _ :<|> _ :<|> r = adventAPIClient aua yr
+    AoCLeaderboard c  -> let _ :<|> _ :<|> _ :<|> _ :<|> _ :<|> r = adventAPIClient aua yr
                          in  r (PublicCode c)
-    AoCDailyLeaderboard d -> let _ :<|> _ :<|> _ :<|> r :<|> _ = adventAPIClient aua yr
+    AoCDailyLeaderboard d -> let _ :<|> _ :<|> _ :<|> _ :<|> r :<|> _ = adventAPIClient aua yr
                              in  r d
-    AoCGlobalLeaderboard  -> let _ :<|> _ :<|> r :<|> _ = adventAPIClient aua yr
+    AoCGlobalLeaderboard  -> let _ :<|> _ :<|> _ :<|> r :<|> _ :<|> _ = adventAPIClient aua yr
                              in  r
-    AoCNextDayTime        -> let r :<|> _ = adventAPIClient aua yr
+    AoCNextDayTime        -> let r :<|> _ :<|> _ :<|> _ :<|> _ :<|> _ = adventAPIClient aua yr
+                             in  r
+    AoCStats              -> let _ :<|> r :<|> _ :<|> _ :<|> _ :<|> _ = adventAPIClient aua yr
                              in  r
 
 
@@ -363,6 +385,7 @@ apiCache sess yr = \case
     AoCDailyLeaderboard d  -> Just $ printf "daily/%04d/%02d.json" yr (dayInt d)
     AoCGlobalLeaderboard{} -> Just $ printf "global/%04d.json" yr
     AoCNextDayTime         -> Nothing
+    AoCStats               -> Nothing
   where
     keyDir = case sess of
       Nothing -> ""
@@ -375,7 +398,7 @@ apiCache sess yr = \case
 -- of leading and trailing whitespace and run through 'URI.encode'
 -- before submitting.
 runAoC :: AoCOpts -> AoC a -> IO (Either AoCError a)
-runAoC AoCOpts{..} a = do
+runAoC opts@AoCOpts{..} a = do
     (keyMayb, cacheDir) <- case _aCache of
       Just c  -> pure (Nothing, c)
       Nothing -> (Just _aSessionKey,) . (</> "advent-of-code-api") <$> getTemporaryDirectory
@@ -406,7 +429,12 @@ runAoC AoCOpts{..} a = do
            . throttling aocThrottler (max 1000000 _aThrottle)
            $ runClientM (aocReq (Just _aUserAgent) _aYear a) =<< aocClientEnv _aSessionKey
       mcr <- maybe (throwError AoCThrottleError) pure mtr
-      either (throwError . AoCClientError) pure mcr
+      res <- either (throwError . AoCClientError) pure mcr
+      case (a, _aInferRankOnSubmission, res) of
+        (AoCSubmit d p _, True, (txt, sr)) -> do
+          sr' <- liftIO $ inferSubmitRes_ opts d p sr
+          pure (txt, sr')
+        _ -> pure res
 
 -- | A version of 'runAoC' that throws an IO exception (of type 'AoCError')
 -- upon failure, instead of an 'Either'.
@@ -415,6 +443,48 @@ runAoC AoCOpts{..} a = do
 runAoC_ :: AoCOpts -> AoC a -> IO a
 runAoC_ o = either throwIO pure <=< runAoC o
 
+-- | Extract the relevant completion count for a given part on a day's stats.
+--
+-- @since 0.2.11.0
+statsForDayPart :: Day -> Part -> Stats -> Integer
+statsForDayPart d Part1 = maybe 0 dsSilver . M.lookup d
+statsForDayPart d Part2 = maybe 0 dsGold   . M.lookup d
+
+-- | If an Advent of Code submission returns 'SubCorrect' without an attached
+-- rank, attempt to infer the rank from 'AoCStats', assuming that it was just
+-- submitted and received at this moment.
+--
+-- If the response is anything other than 'SubCorrect Nothing', it is returned
+-- unchanged.
+--
+-- @since 0.2.11.0
+inferSubmitRes
+    :: AoCOpts
+    -> Day
+    -> Part
+    -> SubmitRes
+    -> IO (Either AoCError SubmitRes)
+inferSubmitRes opts d p sr = case sr of
+    SubCorrect Nothing -> do
+        stats <- runAoC opts AoCStats
+        pure $ stats >>= \st ->
+          case M.lookup d st of
+            Just _  -> Right . SubCorrect . Just $ statsForDayPart d p st
+            Nothing -> Right $ SubCorrect Nothing
+    _ -> pure $ Right sr
+
+-- | Variant of 'inferSubmitRes' that returns the original response if stats
+-- lookup fails.
+--
+-- @since 0.2.11.0
+inferSubmitRes_
+    :: AoCOpts
+    -> Day
+    -> Part
+    -> SubmitRes
+    -> IO SubmitRes
+inferSubmitRes_ opts d p sr = either (const sr) id <$> inferSubmitRes opts d p sr
+
 aocClientEnv :: String -> IO ClientEnv
 aocClientEnv s = do
     t <- getCurrentTime
@@ -473,6 +543,7 @@ saverLoader validToken evt = \case
             pure $ Right lb
         }
     AoCNextDayTime{} -> noCache
+    AoCStats{} -> noCache
   where
     expectedParts :: Set Part
     expectedParts
